@@ -1,8 +1,12 @@
-use std::{io::Write, thread::sleep, time::Duration};
+use std::{
+    io::{BufWriter, Write},
+    thread::sleep,
+    time::Duration,
+};
 
 use binread::{io::Cursor, BinRead, BinReaderExt};
 use bytemuck::{Pod, Zeroable};
-use serialport::SerialPort;
+use serialport::{SerialPort, UsbPortInfo};
 use slip_codec::SlipDecoder;
 
 use crate::{
@@ -11,6 +15,8 @@ use crate::{
     error::{ConnectionError, Error, ResultExt, RomError, RomErrorKind},
 };
 
+const USB_SERIAL_JTAG_PID: u16 = 0x1001;
+
 #[derive(Debug, Copy, Clone, BinRead)]
 pub struct CommandResponse {
     pub resp: u8,
@@ -23,6 +29,7 @@ pub struct CommandResponse {
 
 pub struct Connection {
     serial: Box<dyn SerialPort>,
+    port_info: UsbPortInfo,
     decoder: SlipDecoder,
 }
 
@@ -36,9 +43,10 @@ struct WriteRegParams {
 }
 
 impl Connection {
-    pub fn new(serial: Box<dyn SerialPort>) -> Self {
+    pub fn new(serial: Box<dyn SerialPort>, port_info: UsbPortInfo) -> Self {
         Connection {
             serial,
+            port_info,
             decoder: SlipDecoder::new(),
         }
     }
@@ -57,18 +65,39 @@ impl Connection {
     }
 
     pub fn reset_to_flash(&mut self, extra_delay: bool) -> Result<(), Error> {
-        self.serial.write_data_terminal_ready(false)?;
-        self.serial.write_request_to_send(true)?;
+        if self.port_info.pid == USB_SERIAL_JTAG_PID {
+            self.serial.write_data_terminal_ready(false)?;
+            self.serial.write_request_to_send(false)?;
 
-        sleep(Duration::from_millis(100));
+            sleep(Duration::from_millis(100));
 
-        self.serial.write_data_terminal_ready(true)?;
-        self.serial.write_request_to_send(false)?;
+            self.serial.write_data_terminal_ready(true)?;
+            self.serial.write_request_to_send(false)?;
 
-        let millis = if extra_delay { 500 } else { 50 };
-        sleep(Duration::from_millis(millis));
+            sleep(Duration::from_millis(100));
 
-        self.serial.write_data_terminal_ready(false)?;
+            self.serial.write_request_to_send(true)?;
+            self.serial.write_data_terminal_ready(false)?;
+            self.serial.write_request_to_send(true)?;
+
+            sleep(Duration::from_millis(100));
+
+            self.serial.write_data_terminal_ready(false)?;
+            self.serial.write_request_to_send(false)?;
+        } else {
+            self.serial.write_data_terminal_ready(false)?;
+            self.serial.write_request_to_send(true)?;
+
+            sleep(Duration::from_millis(100));
+
+            self.serial.write_data_terminal_ready(true)?;
+            self.serial.write_request_to_send(false)?;
+
+            let millis = if extra_delay { 500 } else { 50 };
+            sleep(Duration::from_millis(millis));
+
+            self.serial.write_data_terminal_ready(false)?;
+        }
 
         Ok(())
     }
@@ -101,19 +130,20 @@ impl Connection {
     }
 
     pub fn read_response(&mut self) -> Result<Option<CommandResponse>, Error> {
-        let response = self.read()?;
-        if response.len() < 10 {
-            return Ok(None);
+        match self.read(10)? {
+            None => Ok(None),
+            Some(response) => {
+                let mut cursor = Cursor::new(response);
+                let header = cursor.read_le()?;
+                Ok(Some(header))
+            }
         }
-
-        let mut cursor = Cursor::new(response);
-        let header = cursor.read_le()?;
-
-        Ok(Some(header))
     }
 
     pub fn write_command(&mut self, command: Command) -> Result<(), Error> {
-        let mut encoder = SlipEncoder::new(&mut self.serial)?;
+        self.serial.clear(serialport::ClearBuffer::Input)?;
+        let mut writer = BufWriter::new(&mut self.serial);
+        let mut encoder = SlipEncoder::new(&mut writer)?;
         command.write(&mut encoder)?;
         encoder.finish()?;
         Ok(())
@@ -162,10 +192,14 @@ impl Connection {
         Ok(())
     }
 
-    fn read(&mut self) -> Result<Vec<u8>, Error> {
-        let mut output = Vec::with_capacity(1024);
-        self.decoder.decode(&mut self.serial, &mut output)?;
-        Ok(output)
+    fn read(&mut self, len: usize) -> Result<Option<Vec<u8>>, Error> {
+        let mut tmp = Vec::with_capacity(1024);
+        loop {
+            self.decoder.decode(&mut self.serial, &mut tmp)?;
+            if tmp.len() >= len {
+                return Ok(Some(tmp));
+            }
+        }
     }
 
     pub fn flush(&mut self) -> Result<(), Error> {
